
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "debug.h"
#include "dhcpd.h"
#include "arpping.h"
#include "socket.h"
#include "leases.h"
#include "packet.h"
#include "serverpacket.h"

int num_of_reservedIP=0;
char resrvMacAddr[MAX_RESERVED_MAC][MAX_TOKEN_SIZE];
char resrvIpAddr[MAX_RESERVED_IP][MAX_TOKEN_SIZE];

static void trimMacAddr(char *macAddr)
{
    #define C_MAX_MACADDR_LEN       18
    char temp[C_MAX_MACADDR_LEN];
    char *var = macAddr;
    int i = 0;

    while (*var)
    {
        if (*var != ':')
        {
            temp[i++] = *var;
            if (i == C_MAX_MACADDR_LEN-1)
            break;
        }
            var++;
    }
    temp[i] = '\0';

    strcpy(macAddr, temp);
}

static void HexToAscii (unsigned char *hexKey, unsigned char *asciiKey)
{
    unsigned char tmp[32], Data=0, i;
    for (i=0; i<strlen(hexKey); i++)
    {
        tmp[i] = hexKey[i];
        if (tmp[i] >= '0' && tmp[i] <= '9')
            Data = Data*0x10 + tmp[i] - 0x30;
        else if (tmp[i] >= 'A' && tmp[i] <= 'F')
            Data = Data*0x10 + tmp[i] - 0x37;
        else if (tmp[i] >= 'a' && tmp[i] <= 'f')
        {

            Data = Data*0x10 + tmp[i] - 0x57;
        }
        if ((i % 2) == 1) {
            asciiKey[i/2] = Data;
            Data = 0;
        }
    }
    asciiKey[strlen(hexKey)/2]='\0';
}

static int getTokens(char *str, char *delimiter, char token[][MAX_TOKEN_SIZE], int maxNumToken)
{
    char temp[16*1024];    
    char *field;
    int numToken=0, i, j;
    char *ppLast = 0;

    if (strchr(str, '\n')) *(strchr(str, '\n')) = '\0';
    
    if (str == 0 || str[0] == '\0')
    {
        return 0;
    }
   
    strcpy(temp, str);
    
    for (i=0; i<maxNumToken; i++)
    {
        if (i == 0)
            field = strtok_r(temp, delimiter, &ppLast);
        else 
            field = strtok_r(NULL, delimiter, &ppLast);

        if (field == NULL || field[0] == '\0')
        {
                for (j=i; j<maxNumToken; j++)
                    token[j][0] = '\0';
            break;
        }

        numToken++;
        strcpy(token[i], field);
    }

    return numToken;
}


int getReservedAddr(char reservedMacAddr[][MAX_TOKEN_SIZE], char reservedIpAddr[][MAX_TOKEN_SIZE])
{
#define BUFFER_SIZE     2*1024
    int numReservedMac=0, numReservedIp=0;
    char buffer[BUFFER_SIZE];
    FILE *in;
    
	if (!(in = fopen("/tmp/udhcpd_resrv.conf", "r"))) 
    {
		return 0;
	}    
    
    memset(buffer, 0, BUFFER_SIZE);
    if ( fgets(buffer, BUFFER_SIZE, in) )
    {
        numReservedIp = getTokens(buffer, " ", reservedIpAddr, MAX_RESERVED_IP);
    }
    else
    {
        return 0;
    }

    memset(buffer, 0, BUFFER_SIZE);
    if ( fgets(buffer, BUFFER_SIZE, in) )
    {
        numReservedMac = getTokens(buffer, " ", reservedMacAddr, MAX_RESERVED_MAC);
    }
    else
    {
        return 0;
    }

    if (numReservedMac != numReservedIp)
    {
    }

    return (numReservedMac<numReservedIp ? numReservedMac:numReservedIp);
}

int check_reserved_ip(u_int32_t req_ip, u_int8_t *chaddr)
{
    u_int32_t reserved_ip=0;
    int i=0;


    if ( ntohl(req_ip) == ntohl(server_config.server) )
    {
        return 0;
    }


    for (i=0; i<num_of_reservedIP; i++)
    {
        reserved_ip = inet_addr(resrvIpAddr[i]);

        if ( reserved_ip == /*ntohl*/(req_ip) )
        {
            unsigned char tempMac[32], mac[7];
            sprintf(tempMac, "%s", resrvMacAddr[i]);
            trimMacAddr(tempMac);
            HexToAscii(tempMac, mac);
            if ( memcmp(chaddr, mac, 6) == 0 )
            {
                return 1;
            }

            return 0;
        }
    }

    return 1;
}

u_int32_t find_reserved_ip(u_int8_t *chaddr)
{
    int i;

    for (i=0; i<num_of_reservedIP; i++)
    {
        unsigned char tempMac[32], mac[7];
        sprintf(tempMac, "%s", resrvMacAddr[i]);
        trimMacAddr(tempMac);
        HexToAscii(tempMac, mac);
        if ( memcmp(chaddr, mac, 6) == 0 )
        {
            return ntohl(inet_addr(resrvIpAddr[i]));
        }
    }

    return 0;
}
